from websocket import create_connection
from json import dumps , loads
from random import choice
from rubpy.encryption import Encryption

''' This library was created
with the efforts of Shython,
you can get the tutorials and
documentation of the
rubpy library on the
Telegram channel @rubika_library,
if you want to support us, go to
the SYTHON GitHub, i.e. mine, and
the repository star of this library, Rubpy
Hit and fork,
thanks for you
Shayan Heydari ! '''

class connect(object):
	def __init__(self , auth : str , displayWelcome : bool = True):
		if displayWelcome:
			...
		self.auth : str = auth
		self.wss : str = choice([
		'wss://jsocket2.iranlms.ir:80' ,
		'wss://msocket1.iranlms.ir:80' ,
		'wss://jsocket3.iranlms.ir:80'
		 ])

	async def connection(self) :
		ws = create_connection(self.wss)
		data : str = dumps({
			"api_version" : "4",
			"auth" : self.auth,
			"data_enc" : "",
			"method" : "handShake"
			})
		ws.send(data)
		while 1:
			try:
				yield loads(ws.recv())
			except : ...

class Client(object) :
	def __init__(self , auth : str) :
		self.auth : str = auth
		self.connect = connect(self.auth)
		self.enc = Encryption(self.auth)

	async def handler(self) :
		while 1:
			try:
				async for data in self.connect.connection():
					if data.get('type') == 'messenger':
						updates : dict = loads(self.enc.decrypt(data.get('data_enc'))).get('chat_updates')
						if not updates == None:
							yield updates
			except:
				...


from time import sleep , time
from rubpy.encryption import Encryption
import rubpy.connections
from json import dumps , loads
from time import time
from random import randint , choice
from rubpy.tools import Tools

__version__ : str = '3.0.0'
__author__ : str = 'Shayan Heidari'
__copyright__ : str = 'CopyRight 2022'

class createMethod(object):
	''' This class is for creating and
	returning methods to send requests
	to the Rubik's server '''
	def __init__(self:str , auth:str) -> int:
		self.auth : str = auth
		self.enc : str = Encryption(auth)
		self.web : str = { "app_name"    : "Main", "app_version" : "4.0.7", "platform"    : "Web", "package"     : "web.rubika.ir", "lang_code"   : "fa" } #   rubika web client
	
	async def createMethod(self , Type:int , Method:str , data:dict):
		'''This function is for
		creating data to send
		a request to the Rubik's server.'''
		if Type == 4:
			...
		else:
			if Type == 5:
				return dumps(
				{"api_version" : "5",
				"auth" : self.auth,
				"data_enc" : self.enc.encrypt(
				dumps({
				"method" : Method,
				"input" : data,
				"client" : self.web
				}))}).encode()
	
	
class Rubika(object):
	def __init__(self : str , auth : str , displayWelcome : bool = True) -> int:
		if displayWelcome: 
			text : str = f'This library was created by {__author__}, with versions {__version__} and {__copyright__}.\nLibrary activated ...\n\n'
			for char in text:
				print(char , flush = True , end = '')
				sleep(.01)
		self.auth : str = auth # account auth for connect to rubika server
		self.post : str = rubpy.connections.post
		self.enc : str = Encryption(self.auth)
		self.Method : int = createMethod(self.auth)
		self.url : str = choice([
			'https://messengerg2c26.iranlms.ir' ,
			'https://messengerg2c46.iranlms.ir' ,
			'https://messengerg2c39.iranlms.ir'
			])
		self.Tool = Tools()
	
	async def getChatsUpdate(self:int) -> dict:
		'''This function is for
		receiving the latest messages
		that have been sent to your account...'''
		data : str = await self.Method.createMethod(
		5 
		, 'getChatsUpdates', 
		{"state" : str(round(time() - 200))}
		)
		return loads(self.enc.decrypt(loads(await self.post(self.url , data)).get('data_enc'))).get('data').get('chats')
	
	async def sendMessage(self : str , chat_id : str , text : str , metaData = None , message_id : bool = None) -> dict:
		'''Using this function, you can send
		a text message to the desired
		chat, in the first argument,
		you must enter the chat ID where
		the message is to be sent (GUID)
		and in the second argument,
		enter your message as a string
		in the last argument.
		That is, thirdly, if you are going
		to reply to the message,
		click on the message ID,
		the message that is going
		to be replied to (not required).'''
		Input : dict = {
			"object_guid" : chat_id,
			"rnd" : f"{randint(100000,999999999)}",
			"text" : text,
			"reply_to_message_id" : message_id
		}
		if metaData != None:
			Input['metadata'] = {'meta_data_parts' : metaData}
		mode : list = ['**' , '__' , '``']
		for check in mode:
			if check in text:
				metadata : list = self.Tool.textAnalysis(text)
				Input['metadata'] = {'meta_data_parts' : metadata[0]}
				Input['text'] = metadata[1]
				break
		data : dict = await self.Method.createMethod(5 , 'sendMessage', Input)
		return loads(self.enc.decrypt(loads(await self.post(self.url , data)).get('data_enc')))

	async def editMessage(self , chat_id : str , message_id : str , text : str , metaData = None) -> dict:
		'''This method is for editing messages and working with this method is very simple; Enter the chat ID in the first argument, the message ID in the second argument, and the new text in the third argument...'''
		Input : dict = {
			"message_id" : message_id,
			"object_guid" : chat_id,
			"text" : text,
		}
		mode : list = ['**' , '__' , '``']
		for check in mode:
			if check in text:
				metadata : list = self.Tool.textAnalysis(text)
				Input['metadata'] = {'meta_data_parts' : metadata[0]}
				Input['text'] = metadata[1]
				break
		if metaData != None:
			Input['metadata'] = {'meta_data_parts' : metaData}
		data : dict = await self.Method.createMethod(5 , 'editMessage', Input)
		return loads(self.enc.decrypt(loads(await self.post(self.url , data)).get('data_enc')))

	async def banGroupMember(self : str , group_guid : str , user_guid : str ,) -> dict:
		'''You can remove a person
		from your group by using
		this function, just enter the
		group ID (GUID) in the first
		argument and the user ID in
		the second argument.'''
		# bot.banGroupMember('Group Guid' , 'User Guid')
		data : str = await self.Method.createMethod(
			5 
			, 'banGroupMember', 
			{
			'action' : 'Set',
			'group_guid' : group_guid,
			'member_guid' : user_guid
		})
		return loads(self.enc.decrypt(loads(await self.post(self.url , data)).get('data_enc')))
	
class client(object):
	def __init__(self : int , auth : str) -> int:
		try:
			open('answered.txt', 'r').read()
		except FileNotFoundError:
			open('answered.txt', 'w').write('created By Shtyhon :)')
	
		self.auth : str = auth
		self.bot : str = Rubika(self.auth)
	
	async def chats(self : str) -> dict:
		'''This function in the client class , is
		a handler to receive the
		latest messages using
		the getChatsUpdate method.'''
		while 1:
			try:
				chats : str = await self.bot.getChatsUpdate()
				for chat in chats:
					if not chat['object_guid']+chat['last_message']['message_id'] in open('answered.txt', 'r').read().split('\n'):
						yield chat
						open('answered.txt','a+').write('\n'+chat['object_guid']+chat['last_message']['message_id'])
			except :
				...


from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad
from base64 import b64encode , urlsafe_b64decode

class Encryption(object):
	def __init__(self , auth : str) -> int:
		self.key : str = bytearray(self.secret(auth), "UTF-8")
		self.iv : str = bytearray.fromhex('0'*32)

	def replaceCharAt(self , e : str, t : str, i : str) -> str:
		return e[0:t] + i + e[t + len(i):]

	def secret(self , e : str):
		t : str = e[0:8]
		i : str = e[8:16]
		n : str = e[16:24] + t + e[24:32] + i
		s : int = 0
		while (s < len(n)):
			e : str = n[s]
			if e >= '0' and e <= '9':
				t : str = chr((ord(e[0]) - ord('0') + 5) % 10 + ord('0'))
				n : str = self.replaceCharAt(n, s, t)
			else:
				t:str = chr((ord(e[0]) - ord('a') + 9) % 26 + ord('a'))
				n : str = self.replaceCharAt(n, s, t)
			s += 1
		return n

	def encrypt(self , text : str) -> str:
		return b64encode(AES.new(self.key,
		AES.MODE_CBC, self.iv).encrypt(
		pad(text.encode('UTF-8'),
		AES.block_size))).decode('UTF-8')

	def decrypt(self , text : str) -> str:
		return unpad(AES.new(self.key,
		AES.MODE_CBC, self.iv).decrypt(
		urlsafe_b64decode(
		text.encode('UTF-8'))),
		AES.block_size).decode('UTF-8')


from PIL import Image
from base64 import b64encode
from io import BytesIO

class media(object):
	def __init__(self) -> int:
		...

	async def getImageThumbnail(self , image_bytes : bytes) -> bytes:
		im = Image.open(BytesIO(image_bytes))
		[width , height] = im.size
		if height > width:
			new_height : int = 40
			new_width  = round(new_height * width / height)
		else:
			new_width : int = 40
			new_height : int = round(new_width * height / width)
			im = im.resize((new_width , new_height), Image.ANTIALIAS)
			changed_image : bytes = BytesIO()
			im.save(changed_image , format='PNG')
			changed_image : bytes = changed_image.getvalue()
		return b64encode(changed_image)


from aiohttp import ClientSession
from json import dumps , loads

"""This library was created by shayan heidari....
And This library works with the help of AIOhttp"""

async def get(url:str) -> str:
	async with ClientSession() as session:
		async with session.get(url) as _:
			return _.text()

async def post(url:str , data:dict) -> str:
	async with ClientSession() as _:
		async with _.post(url, data=data) as post:
			return await post.text()


from re import findall

class Tools(object):
	def __init__(self) -> int:
		...

	def textAnalysis(self , text : str) -> list:
		Results : list = []

		realText : str = text.replace("**","").replace("__","").replace("``","")
		bolds : list = findall(r"\*\*(.*?)\*\*" , text)
		italics : list = findall(r"\_\_(.*?)\_\_" , text)
		monos : list = findall(r"\`\`(.*?)\`\`" , text)

		bResult : list = [realText.index(i) for i in bolds]
		iResult : list = [realText.index(i) for i in italics]
		mResult : list = [realText.index(i) for i in monos]

		for bIndex , bWord in zip(bResult , bolds):
			Results.append({
				"from_index" : bIndex,
				"length" : len(bWord),
				"type" : "Bold"
			})

		for iIndex , iWord in zip(iResult , italics):
			Results.append({
				"from_index" : iIndex,
				"length" : len(iWord),
				"type" : "Italic"
			})

		for mIndex , mWord in zip(mResult , monos):
			Results.append({
				"from_index" : mIndex,
				"length" : len(mWord),
				"type" : "Mono"
			})

		return Results , realText


from setuptools import setup, find_packages

with open("README.md", "r") as readme_file:
    readme : str = readme_file.read()

requirements : list = ["pycryptodome" , "pillow" , "aiohttp" , "websocket-client"]

setup(
    name = "rubpy",
    version = "2.0.5",
    author = "Shayan Heidari",
    author_email = "snipe4kill.tg@gmail.com",
    description = "This is an unofficial library for deploying robots on Rubika accounts.",
    long_description = readme,
    long_description_content_type = "text/markdown",
    url = "https://github.com/snipe4kill/rubpy/",
    packages = find_packages(),
    install_requires = requirements,
    classifiers = [
    	"Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
    ],
)